import fastify from 'fastify';
import cors from '@fastify/cors';
import fastifySensible from '@fastify/sensible';
import helmet from '@fastify/helmet';

import validator from './plugins/validator';
import { mainHandler, relayerHandler } from './routes';

function createServer() {
  const server = fastify({
    logger: true,
    trustProxy: true,
    pluginTimeout: 5000,
    bodyLimit: 1048576, // 1 mb
  });
  server.register(cors);
  server.register(validator);
  server.register(helmet, { contentSecurityPolicy: false, frameguard: true });
  server.register(fastifySensible);
  server.register(mainHandler);
  server.register(mainHandler, { prefix: '/v1' });
  server.register(relayerHandler, { prefix: '/v1' });

  server.setErrorHandler((error, req, res) => {
    req.log.error(error.toString());
    let statusCode = 500;
    if ('validation' in error) {
      statusCode = 400;
    }
    res.code(statusCode).send({ error: error.toString() });
  });
  return server;
}

export default createServer;
